#include <stdio.h>
#include "../version.h"
#include "driver.h"

char * camlrun = "camlrun.exe";
char * stdlib;
int linkalso = 1;
char * includes = "";
char * compopt = "";
char * compfiles = "";
char * linkopt = "";
char * linkfiles = "";
char * linkout = "camlout.exe";
int custom = 0;
char * ccfiles = "";
char * ccopt = "";

main(argc, argv)
     int argc;
     char ** argv;
{
  int i;
  char * a;
  extern char * getenv();
  char * cmd;
  int status;
  char * lang;
  
  stdlib = getenv("CAMLLIB");
  if (stdlib == NULL) {
    fprintf(stderr, "Variable CAMLLIB is undefined.\n");
    exit(2);
  }

  lang = getenv("LANG");
  if (lang != NULL) {
    reformat(&compopt, "%s -lang %s", compopt, lang);
    reformat(&linkopt, "%s -lang %s", linkopt, lang);
  }

  for (i = 1; i < argc; i++) {
    a = argv[i];
    if (suffix(a, ".ml")) {
      reformat(&compfiles, "%s %s", compfiles, a);
      reformat(&linkfiles, "%s %s", linkfiles, a);
    } else
    if (suffix(a, ".mli")) {
      reformat(&compfiles, "%s %s", compfiles, a);
    } else
    if (suffix(a, ".zo")) {
      reformat(&linkfiles, "%s %s", linkfiles, a);
    } else
    if (eq(a, "-c")) {
      linkalso = 0;
    } else
    if (eq(a, "-I") || eq(a, "-include")) {
      reformat(&includes, "%s -I %s", includes, argv[++i]);
    } else
    if (eq(a, "-O") || eq(a, "-open")) {
      reformat(&compopt, "%s -O %s", compopt, argv[++i]);
    } else
    if (eq(a, "-i")) {
      reformat(&compopt, "%s -i", compopt);
    } else
    if (eq(a, "-g") || eq(a, "-debug")) {
      reformat(&linkopt, "%s -g", linkopt);
      reformat(&compopt, "%s -g", compopt);
    } else
    if (eq(a, "-o") || eq(a, "-exec")) {
      linkout = argv[++i];
    } else
    if (eq(a, "-W")) {
      reformat(&compopt, "%s -W", compopt);
    } else
    if (eq(a, "-stdlib")) {
      stdlib = argv[++i];
    } else
    if (eq(a, "-v") || eq(a, "-version")) {
      printf("The Caml Light system for the 80386 PC, version %s\n",
             VERSION);
      printf("  (standard library from %s)\n", stdlib);
      execute(camlrun, "-V");
      format(&cmd, "%s\\camlcomp -version", stdlib);
      execute(camlrun, cmd);
      free(cmd);
      format(&cmd, "%s\\camllink -version", stdlib);
      execute(camlrun, cmd);
      free(cmd);
    } else
    if (eq(a, "-custom")) {
      custom = 1;
    } else
    if (suffix(a, ".c")) {
      format(&cmd, "-c -I%s %s %s", stdlib, ccopt, a);
      status = execute("gcc", cmd);
      if (status != 0) exit(status);
      free(cmd);
      a[strlen(a) - 2] = 0;     /* remove .c extension */
      reformat(&ccfiles, "%s %s.o", ccfiles, a);
    } else
    if (suffix(a, ".o") || suffix(a, ".a") || prefix(a, "-l")) {
      reformat(&ccfiles, "%s %s", ccfiles, a);
    } else
    if (eq(a, "-ccopt")) {
      reformat(&ccopt, "%s %s", ccopt, argv[++i]);
    } else
    if (prefix(a, "-")) {
      fprintf(stderr, "Unknown option \"%s\", ignored\n", a);
    } else {
      fprintf(stderr, "I don't know what to do with file \"%s\", ignored\n",
              a);
    }
  }
  if (compfiles[0] != 0) {
    format(&cmd, "%s\\camlcomp -stdlib %s %s %s %s",
           stdlib, stdlib, includes, compopt, compfiles);
    status = execute(camlrun, cmd);
    if (status != 0) exit(status);
    free(cmd);
  }
  if (linkalso && linkfiles[0] != 0) {
    if (!custom) {
      format(&cmd,
             "%s\\camllink -stdlib %s %s %s -exec %s %s\\stdlib.zo %s",
             stdlib, stdlib, includes,
             linkopt, linkout, stdlib, linkfiles);
      terminate(camlrun, cmd);
    } else {
      static char temp_out[] = "camlXXXX.byt";
      static char prim_file[] = "primXXXX.c";
      char * gcc_out, * exefile;
      tempname(temp_out);
      tempname(prim_file);
      gcc_out = change_suffix(linkout, ".cff");
      exefile = change_suffix(gcc_out, ".exe");

      format(&cmd,
             "%s\\camllink -stdlib %s -custom %s %s %s -exec %s %s\\stdlib.zo %s",
             stdlib, stdlib, prim_file, includes,
             linkopt, temp_out, stdlib, linkfiles);
      status = execute(camlrun, cmd);
      free(cmd);
      if (status != 0) goto end;
      format(&cmd,
             "-I%s -o %s %s %s -L%s %s -lcaml -lgrx -lpc -lm",
             stdlib, gcc_out, ccopt, prim_file, stdlib, ccfiles);
      status = execute("gcc", cmd);
      free(cmd);
      if (status != 0) goto end;
      format(&cmd, "-s %s\\go32.exe %s", stdlib, gcc_out);
      status = execute("coff2exe", cmd);
      free(cmd);
      if (status != 0) goto end;
      status = append(temp_out, exefile);
      free(cmd);
    end:
      unlink(temp_out);
      unlink(prim_file);
      unlink(gcc_out);
      if (status != 0) unlink(exefile);
      exit(status);
    }
  }
  return 0;
}
